@@ -30,17 +30,16 @@ from salt.ext import six
 from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
 
 from tests.support.unit import TestCase
-from tests.support.helpers import win32_kill_process_tree
-from tests.support.paths import CODE_DIR
+from tests.support.runtests import RUNTIME_VARS
 from tests.support.processes import terminate_process, terminate_process_list
-
+from tests.support.cli_scripts import ScriptPathMixin
 log = logging.getLogger(__name__)
 
 
 if 'TimeoutError' not in __builtins__:
     class TimeoutError(OSError):
         '''Compatibility exception with python3'''
-        pass
+
     __builtins__['TimeoutError'] = TimeoutError
 
 
@@ -389,8 +388,8 @@ class TestProgram(six.with_metaclass(TestProgramMeta, object)):
                     if path not in env_pypath:
                         env_pypath.append(path)
             # Always ensure that the test tree is searched first for python modules
-            if CODE_DIR != env_pypath[0]:
-                env_pypath.insert(0, CODE_DIR)
+            if RUNTIME_VARS.CODE_DIR != env_pypath[0]:
+                env_pypath.insert(0, RUNTIME_VARS.CODE_DIR)
             if salt.utils.platform.is_windows():
                 env_delta['PYTHONPATH'] = ';'.join(env_pypath)
             else:
@@ -440,27 +439,8 @@ class TestProgram(six.with_metaclass(TestProgramMeta, object)):
                 process.poll()
 
                 if datetime.now() > stop_at:
-                    if term_sent is False:
-                        if salt.utils.platform.is_windows():
-                            _, alive = win32_kill_process_tree(process.pid)
-                            if alive:
-                                log.error("Child processes still alive: %s", alive)
-                        else:
-                            # Kill the process group since sending the term signal
-                            # would only terminate the shell, not the command
-                            # executed in the shell
-                            os.killpg(os.getpgid(process.pid), signal.SIGINT)
-                            term_sent = True
-                            continue
-
                     try:
-                        if salt.utils.platform.is_windows():
-                            _, alive = win32_kill_process_tree(process.pid)
-                            if alive:
-                                log.error("Child processes still alive: %s", alive)
-                        else:
-                            # As a last resort, kill the process group
-                            os.killpg(os.getpgid(process.pid), signal.SIGKILL)
+                        terminate_process(pid=process.pid, kill_children=True)
                         process.wait()
                     except OSError as exc:
                         if exc.errno != errno.ESRCH:
@@ -592,7 +572,7 @@ class TestSaltProgramMeta(TestProgramMeta):
         return super(TestSaltProgramMeta, mcs).__new__(mcs, name, bases, attrs)
 
 
-class TestSaltProgram(six.with_metaclass(TestSaltProgramMeta, TestProgram)):
+class TestSaltProgram(six.with_metaclass(TestSaltProgramMeta, TestProgram, ScriptPathMixin)):
     '''
     This is like TestProgram but with some functions to run a salt-specific
     auxiliary program.
@@ -643,9 +623,7 @@ class TestSaltProgram(six.with_metaclass(TestSaltProgramMeta, TestProgram)):
             # This is effectively a place-holder - it gets set correctly after super()
             kwargs['program'] = self.script
         super(TestSaltProgram, self).__init__(*args, **kwargs)
-        self.program = self.abs_path(os.path.join(self.script_dir, self.script))
-        path = self.env.get('PATH', os.getenv('PATH'))
-        self.env['PATH'] = ':'.join([self.abs_path(self.script_dir), path])
+        self.program = self.get_script_path(self.script)
 
     def config_merge(self, base, overrides):
         _base = self.config_cast(copy.deepcopy(base))
@@ -687,29 +665,7 @@ class TestSaltProgram(six.with_metaclass(TestSaltProgramMeta, TestProgram)):
                 cfg[key] = val
         return salt.utils.yaml.safe_dump(cfg, default_flow_style=False)
 
-    def setup(self, *args, **kwargs):
-        super(TestSaltProgram, self).setup(*args, **kwargs)
-        self.install_script()
-
-    def install_script(self):
-        '''Generate the script file that calls python objects and libraries.'''
-        lines = []
-        script_source = os.path.join(CODE_DIR, 'scripts', self.script)
-        with salt.utils.files.fopen(script_source, 'r') as sso:
-            lines.extend(sso.readlines())
-        if lines[0].startswith('#!'):
-            lines.pop(0)
-        lines.insert(0, '#!{0}\n'.format(sys.executable))
-
-        script_path = self.abs_path(os.path.join(self.script_dir, self.script))
-        log.debug('Installing "{0}" to "{1}"'.format(script_source, script_path))
-        with salt.utils.files.fopen(script_path, 'w') as sdo:
-            sdo.write(''.join(lines))
-            sdo.flush()
-
-        os.chmod(script_path, 0o755)
-
-    def run(self, **kwargs):
+    def run(self, **kwargs):  # pylint: disable=arguments-differ
         if not kwargs.get('verbatim_args'):
             args = kwargs.setdefault('args', [])
             if '-c' not in args and '--config-dir' not in args:
@@ -890,7 +846,6 @@ class TestSaltDaemon(six.with_metaclass(TestSaltProgramMeta, TestDaemon, TestSal
     '''
     A class to run arbitrary salt daemons (master, minion, syndic, etc.)
     '''
-    pass
 
 
 class TestDaemonSaltMaster(TestSaltDaemon):
@@ -923,7 +878,6 @@ class TestDaemonSaltApi(TestSaltDaemon):
     '''
     Manager for salt-api daemon.
     '''
-    pass
 
 
 class TestDaemonSaltSyndic(TestSaltDaemon):
